/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 1999 by Pedro Pombeiro
// All rights reserved
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc., and
// I'll try to keep a version up to date.  I can be reached as follows:
//    PPombeiro@AntaresLda.pt         (company mail account)
/////////////////////////////////////////////////////////////////////////////


// SelectionSliderCtrl.cpp : implementation file
//

#include "StdAFX.h"
#include "SelectionSliderCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CSelectionSliderCtrl

CSelectionSliderCtrl::CSelectionSliderCtrl()
{
	m_bSelecting		= FALSE;
	m_bAnchorIsValid	= FALSE;
	m_iAnchorSelection	= -1;
}

CSelectionSliderCtrl::~CSelectionSliderCtrl()
{
}


BEGIN_MESSAGE_MAP(CSelectionSliderCtrl, CSliderCtrl)
	//{{AFX_MSG_MAP(CSelectionSliderCtrl)
	ON_WM_HSCROLL_REFLECT()
	ON_WM_VSCROLL_REFLECT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSelectionSliderCtrl message handlers

void CSelectionSliderCtrl::HScroll(UINT nTBCode, UINT /*nPos*/)
{
	if (GetStyle() & TBS_ENABLESELRANGE)
	{
		BOOL	bEndScroll	= FALSE;
		int		iPos		= GetPos();
		int		iPrevPos	= iPos;

		switch (nTBCode)
		{
			case TB_PAGEUP		: iPrevPos	= iPos + GetPageSize(); break;
			case TB_PAGEDOWN	: iPrevPos	= iPos - GetPageSize(); break;
			case TB_LINEUP		: iPrevPos	= iPos + GetLineSize(); break;
			case TB_LINEDOWN	: iPrevPos	= iPos - GetLineSize(); break;
		}

		switch (nTBCode)
		{
			case TB_ENDTRACK		:
				bEndScroll	= TRUE;
				break;
			case TB_PAGEUP			:
			case TB_PAGEDOWN		:
			case TB_LINEUP			:
			case TB_LINEDOWN		:
			case TB_THUMBPOSITION	:
			case TB_THUMBTRACK		:
			{
				const BOOL	bShiftIsPressed	= (::GetKeyState(VK_SHIFT) < 0);

				if (!m_bSelecting && !bShiftIsPressed)
				{
					m_iAnchorSelection	= -1;
					m_bAnchorIsValid	= FALSE;
				}
				if (!m_bSelecting && !m_bAnchorIsValid)
				{
					int	iMin, iMax;
					GetSelection(iMin, iMax);
					if (iMin != -1 || iMax != -1)
						ClearSel(TRUE);
				}
				if (!m_bSelecting && bShiftIsPressed)
				{
					m_bSelecting	= TRUE;
					if (!m_bAnchorIsValid)
					{
						m_iAnchorSelection	= iPrevPos;
						m_bAnchorIsValid	= TRUE;
					}
				}
				if (m_bSelecting)
				{
					SetSelection(min(m_iAnchorSelection, iPos), max(m_iAnchorSelection, iPos));
					Invalidate(FALSE);
				}
				break;
			}
			default	:
				ASSERT(FALSE);
				break;
		}

		if (bEndScroll)
		{
			if (!m_bSelecting || ::GetKeyState(VK_SHIFT) >= 0)
			{
				m_bAnchorIsValid	= FALSE;
				m_iAnchorSelection	= -1;
			}
			m_bSelecting	= FALSE;
		}
	}
}

void CSelectionSliderCtrl::VScroll(UINT nTBCode, UINT nPos)
{
	HScroll(nTBCode, nPos);
}
